@@ -20,7 +20,7 @@
 #include <afs/param.h>
 
 RCSID
-    ("$Header: /cvs/openafs/src/viced/viced.c,v 1.58.2.5 2005/05/30 04:41:28 shadow Exp $");
+    ("$Header: /cvs/openafs/src/viced/viced.c,v 1.58.2.19.2.1 2007/12/13 21:00:43 shadow Exp $");
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -28,8 +28,8 @@ RCSID
 #include <sys/types.h>
 #include <afs/procmgmt.h>	/* signal(), kill(), wait(), etc. */
 #include <sys/stat.h>
-#ifdef AFS_NT40_ENV
 #include <fcntl.h>
+#ifdef AFS_NT40_ENV
 #include <io.h>
 #include <windows.h>
 #include <WINNT/afsevent.h>
@@ -88,7 +88,6 @@ RCSID
 #include "host.h"
 #ifdef AFS_PTHREAD_ENV
 #include "softsig.h"
-char *(*threadNameProgram) ();
 #endif
 #if defined(AFS_SGI_ENV)
 #include "sys/schedctl.h"
@@ -151,8 +150,6 @@ struct afsconf_dir *confDir;	/* Configuration dir object */
 
 int restartMode = RESTART_ORDINARY;
 
-int Testing = 0;		/* for ListViceInodes */
-
 /*
  * Home for the performance statistics.
  */
@@ -168,6 +165,8 @@ int SawPctSpare;
 int debuglevel = 0;
 int printBanner = 0;
 int rxJumbograms = 1;		/* default is to send and receive jumbograms. */
+int rxBind = 0;		/* don't bind */
+int rxMaxMTU = -1;
 afs_int32 implicitAdminRights = PRSFS_LOOKUP;	/* The ADMINISTER right is 
 						 * already implied */
 afs_int32 readonlyServer = 0;
@@ -197,6 +196,10 @@ int sendBufSize = 16384;	/* send buffer size */
 
 struct timeval tp;
 
+#ifdef AFS_PTHREAD_ENV
+pthread_key_t viced_uclient_key;
+#endif
+
 /*
  * FileServer's name and IP address, both network byte order and
  * host byte order.
@@ -318,14 +321,9 @@ ResetCheckDescriptors(void)
 
 #if defined(AFS_PTHREAD_ENV)
 char *
-threadName(void)
+threadNum(void)
 {
-    char threadid[16];
-    if (LogLevel > 999) {
-	afs_snprintf(threadid, 16, "%d", pthread_getspecific(rx_thread_id_key));
-	return threadid;
-    } else 
-	return NULL;
+    return pthread_getspecific(rx_thread_id_key);
 }
 #endif
 
@@ -342,8 +340,10 @@ get_key(char *arock, register afs_int32 akvno, char *akey)
 	return 1;
     }
     code = afsconf_GetKey(confDir, akvno, tkey.key);
-    if (code)
+    if (code) {
+	ViceLog(0, ("afsconf_GetKey failure: kvno %d code %d\n", akvno, code));
 	return code;
+    }
     memcpy(akey, tkey.key, sizeof(tkey.key));
     return 0;
 
@@ -699,6 +699,9 @@ ShutDownAndCore(int dopanic)
 	}
     }
 
+    if (dopanic)
+	assert(0);
+
     exit(0);
 
 }				/*ShutDown */
@@ -718,6 +721,7 @@ FlagMsg()
     /* default supports help flag */
 
     strcpy(buffer, "Usage: fileserver ");
+    strcpy(buffer, "[-auditlog <log path>] ");
     strcat(buffer, "[-d <debug level>] ");
     strcat(buffer, "[-p <number of processes>] ");
     strcat(buffer, "[-spare <number of spare blocks>] ");
@@ -739,6 +743,8 @@ FlagMsg()
     strcat(buffer, "[-rxpck <number of rx extra packets>] ");
     strcat(buffer, "[-rxdbg (enable rx debugging)] ");
     strcat(buffer, "[-rxdbge (enable rxevent debugging)] ");
+    strcat(buffer, "[-rxmaxmtu <bytes>] ");
+    strcat(buffer, "[-rxbind (bind the Rx socket to one address)] ");
 #if AFS_PTHREAD_ENV
     strcat(buffer, "[-vattachpar <number of volume attach threads>] ");
 #endif
@@ -754,6 +760,7 @@ FlagMsg()
     strcat(buffer, "[-realm <Kerberos realm name>] ");
     strcat(buffer, "[-udpsize <size of socket buffer in bytes>] ");
     strcat(buffer, "[-sendsize <size of send buffer in bytes>] ");
+    strcat(buffer, "[-abortthreshold <abort threshold>] ");
 /*   strcat(buffer, "[-enable_peer_stats] "); */
 /*   strcat(buffer, "[-enable_process_stats] "); */
     strcat(buffer, "[-help]\n");
@@ -1054,6 +1061,21 @@ ParseArgs(int argc, char *argv[])
 #endif
 	else if (!strcmp(argv[i], "-nojumbo")) {
 	    rxJumbograms = 0;
+	} else if (!strcmp(argv[i], "-rxbind")) {
+	    rxBind = 1;
+	} else if (!strcmp(argv[i], "-rxmaxmtu")) {
+	    if ((i + 1) >= argc) {
+		fprintf(stderr, "missing argument for -rxmaxmtu\n"); 
+		return -1; 
+	    }
+	    rxMaxMTU = atoi(argv[++i]);
+	    if ((rxMaxMTU < RX_MIN_PACKET_SIZE) || 
+		(rxMaxMTU > RX_MAX_PACKET_DATA_SIZE)) {
+		printf("rxMaxMTU %d% invalid; must be between %d-%d\n",
+		       rxMaxMTU, RX_MIN_PACKET_SIZE, 
+		       RX_MAX_PACKET_DATA_SIZE);
+		return -1;
+	    }
 	} else if (!strcmp(argv[i], "-realm")) {
 	    extern char local_realm[AFS_REALM_SZ];
 	    if ((i + 1) >= argc) {
@@ -1096,6 +1118,36 @@ ParseArgs(int argc, char *argv[])
 	} else if (!strcmp(argv[i], "-enable_process_stats")) {
 	    rx_enableProcessRPCStats();
 	}
+	else if (strcmp(argv[i], "-auditlog") == 0) {
+	    int tempfd, flags;
+	    FILE *auditout;
+	    char oldName[MAXPATHLEN];
+	    char *fileName = argv[++i];
+	    
+#ifndef AFS_NT40_ENV
+	    struct stat statbuf;
+	    
+	    if ((lstat(fileName, &statbuf) == 0) 
+		&& (S_ISFIFO(statbuf.st_mode))) {
+		flags = O_WRONLY | O_NONBLOCK;
+	    } else 
+#endif
+	    {
+		strcpy(oldName, fileName);
+		strcat(oldName, ".old");
+		renamefile(fileName, oldName);
+		flags = O_WRONLY | O_TRUNC | O_CREAT;
+	    }
+	    tempfd = open(fileName, flags, 0666);
+	    if (tempfd > -1) {
+		auditout = fdopen(tempfd, "a");
+		if (auditout) {
+		    osi_audit_file(auditout);
+		} else
+		    printf("Warning: auditlog %s not writable, ignored.\n", fileName);
+	    } else
+		printf("Warning: auditlog %s not writable, ignored.\n", fileName);
+	}
 #ifndef AFS_NT40_ENV
 	else if (strcmp(argv[i], "-syslog") == 0) {
 	    /* set syslog logging flag */
@@ -1105,6 +1157,10 @@ ParseArgs(int argc, char *argv[])
 	    serverLogSyslogFacility = atoi(argv[i] + 8);
 	}
 #endif
+	else if (strcmp(argv[i], "-mrafslogs") == 0) {
+	    /* set syslog logging flag */
+	    mrafsStyleLogs = 1;
+	} 
 	else {
 	    return (-1);
 	}
@@ -1247,6 +1303,11 @@ InitPR()
 		("Couldn't initialize protection library; code=%d.\n", code));
 	return code;
     }
+
+#ifdef AFS_PTHREAD_ENV
+    assert(pthread_key_create(&viced_uclient_key, NULL) == 0);
+#endif
+
     SystemId = SYSADMINID;
     SystemAnyUser = ANYUSERID;
     SystemAnyUserCPS.prlist_len = 0;
@@ -1314,6 +1375,7 @@ vl_Initialize(char *confDir)
 			     info.hostAddr[i].sin_port, USER_SERVICE_ID, sc,
 			     scIndex);
     code = ubik_ClientInit(serverconns, &cstruct);
+    afsconf_Close(tdir);
     if (code) {
 	ViceLog(0, ("vl_Initialize: ubik client init failed.\n"));
 	return code;
@@ -1385,16 +1447,22 @@ ReadSysIdFile()
 		 AFSDIR_SERVER_SYSID_FILEPATH, nentries));
 	return EIO;
     }
-    FS_HostAddr_cnt = nentries;
-    for (i = 0; i < nentries; i++) {
-	if (read(fd, (char *)&FS_HostAddrs[i], sizeof(afs_int32)) !=
-	    sizeof(afs_int32)) {
-	    ViceLog(0,
-		    ("%s: Read of addresses failed (%d)\n",
-		     AFSDIR_SERVER_SYSID_FILEPATH, errno));
-	    FS_HostAddr_cnt = 0;	/* reset it */
-	    return EIO;
+    if (FS_HostAddr_cnt == 0) {
+	FS_HostAddr_cnt = nentries;
+	for (i = 0; i < nentries; i++) {
+	    if (read(fd, (char *)&FS_HostAddrs[i], sizeof(afs_int32)) !=
+		sizeof(afs_int32)) {
+		ViceLog(0,
+			("%s: Read of addresses failed (%d)\n",
+			 AFSDIR_SERVER_SYSID_FILEPATH, errno));
+		FS_HostAddr_cnt = 0;	/* reset it */
+		return EIO;
+	    }
 	}
+    } else {
+	ViceLog(1,
+		("%s: address list ignored (NetInfo/NetRestrict override)\n",
+		 AFSDIR_SERVER_SYSID_FILEPATH));
     }
     close(fd);
     return 0;
@@ -1482,7 +1550,7 @@ Do_VLRegisterRPC()
 	FS_HostAddrs_HBO[i] = ntohl(FS_HostAddrs[i]);
     addrs.bulkaddrs_len = FS_HostAddr_cnt;
     addrs.bulkaddrs_val = (afs_uint32 *) FS_HostAddrs_HBO;
-    code = ubik_Call(VL_RegisterAddrs, cstruct, 0, &FS_HostUUID, 0, &addrs);
+    code = ubik_VL_RegisterAddrs(cstruct, 0, &FS_HostUUID, 0, &addrs);
     if (code) {
 	if (code == VL_MULTIPADDR) {
 	    ViceLog(0,
@@ -1507,6 +1575,43 @@ Do_VLRegisterRPC()
     return 0;
 }
 
+afs_int32
+SetupVL()
+{
+    afs_int32 code;
+    extern int rxi_numNetAddrs;
+    extern afs_uint32 rxi_NetAddrs[];
+
+#ifndef AFS_NT40_ENV
+    if (AFSDIR_SERVER_NETRESTRICT_FILEPATH || AFSDIR_SERVER_NETINFO_FILEPATH) {
+	/*
+	 * Find addresses we are supposed to register as per the netrestrict 
+	 * and netinfo files (/usr/afs/local/NetInfo and 
+	 * /usr/afs/local/NetRestict)
+	 */
+	char reason[1024];
+	afs_int32 code = parseNetFiles(FS_HostAddrs, NULL, NULL,
+				       ADDRSPERSITE, reason,
+				       AFSDIR_SERVER_NETINFO_FILEPATH,
+				       AFSDIR_SERVER_NETRESTRICT_FILEPATH);
+	if (code < 0) {
+	    ViceLog(0, ("Can't register any valid addresses: %s\n", reason));
+	    exit(1);
+	}
+	FS_HostAddr_cnt = (afs_uint32) code;
+    } else
+#endif
+    {
+	FS_HostAddr_cnt = rx_getAllAddr(FS_HostAddrs, ADDRSPERSITE);
+    }
+
+    if (FS_HostAddr_cnt == 1 && rxBind == 1)
+	code = FS_HostAddrs[0];
+    else 
+	code = htonl(INADDR_ANY);
+    return code;
+}
+
 afs_int32
 InitVL()
 {
@@ -1540,30 +1645,7 @@ InitVL()
     /* A good sysid file exists; inform the vlserver. If any conflicts,
      * we always use the latest interface available as the real truth.
      */
-#ifndef AFS_NT40_ENV
-    if (AFSDIR_SERVER_NETRESTRICT_FILEPATH || AFSDIR_SERVER_NETINFO_FILEPATH) {
-	/*
-	 * Find addresses we are supposed to register as per the netrestrict 
-	 * and netinfo files (/usr/afs/local/NetInfo and 
-	 * /usr/afs/local/NetRestict)
-	 */
-	char reason[1024];
-	afs_int32 code = parseNetFiles(FS_HostAddrs, NULL, NULL,
-				       ADDRSPERSITE, reason,
-				       AFSDIR_SERVER_NETINFO_FILEPATH,
-				       AFSDIR_SERVER_NETRESTRICT_FILEPATH);
-	if (code < 0) {
-	    ViceLog(0, ("Can't register any valid addresses: %s\n", reason));
-	    exit(1);
-	}
-	FS_HostAddr_cnt = (afs_uint32) code;
-    } else
-#endif
-    {
-	FS_HostAddr_cnt = rx_getAllAddr(FS_HostAddrs, ADDRSPERSITE);
-    }
 
-    FS_registered = 1;
     code = Do_VLRegisterRPC();
     return code;
 }
@@ -1588,6 +1670,7 @@ main(int argc, char *argv[])
 #endif
     int curLimit;
     time_t t;
+    afs_uint32 rx_bindhost;
 
 #ifdef	AFS_AIX32_ENV
     struct sigaction nsa;
@@ -1598,6 +1681,7 @@ main(int argc, char *argv[])
     sigaction(SIGABRT, &nsa, NULL);
     sigaction(SIGSEGV, &nsa, NULL);
 #endif
+    osi_audit_init();
 
     /* Initialize dirpaths */
     if (!(initAFSDirPath() & AFSDIR_SERVER_PATHS_OK)) {
@@ -1726,8 +1810,8 @@ main(int argc, char *argv[])
 	V_BreakVolumeCallbacks = BreakVolumeCallBacksLater;
     }
 
-#if defined(AFS_PTHREAD_ENV)
-    threadNameProgram = threadName;
+#ifdef AFS_PTHREAD_ENV
+    SetLogThreadNumProgram( threadNum );
 #endif
 
     /* initialize libacl routines */
@@ -1752,7 +1836,9 @@ main(int argc, char *argv[])
 #endif
     if (udpBufSize)
 	rx_SetUdpBufSize(udpBufSize);	/* set the UDP buffer size for receive */
-    if (rx_Init((int)htons(7000)) < 0) {
+    rx_bindhost = SetupVL();
+
+    if (rx_InitHost(rx_bindhost, (int)htons(7000)) < 0) {
 	ViceLog(0, ("Cannot initialize RX\n"));
 	exit(1);
     }
@@ -1760,23 +1846,26 @@ main(int argc, char *argv[])
 	/* Don't send and don't allow 3.4 clients to send jumbograms. */
 	rx_SetNoJumbo();
     }
+    if (rxMaxMTU != -1) {
+	rx_SetMaxMTU(rxMaxMTU);
+    }
     rx_GetIFInfo();
     rx_SetRxDeadTime(30);
     sc[0] = rxnull_NewServerSecurityObject();
     sc[1] = 0;			/* rxvab_NewServerSecurityObject(key1, 0) */
     sc[2] = rxkad_NewServerSecurityObject(rxkad_clear, NULL, get_key, NULL);
     sc[3] = rxkad_NewServerSecurityObject(rxkad_crypt, NULL, get_key, NULL);
-    tservice = rx_NewService( /* port */ 0, /* service id */ 1,	/*service name */
-			     "AFS",
-							/* security classes */ sc,
-							/* numb sec classes */
-			     4, RXAFS_ExecuteRequest);
+    tservice = rx_NewServiceHost(rx_bindhost,  /* port */ 0, /* service id */ 
+				 1,	/*service name */
+				 "AFS",
+				 /* security classes */ sc,
+				 /* numb sec classes */
+				 4, RXAFS_ExecuteRequest);
     if (!tservice) {
 	ViceLog(0,
 		("Failed to initialize RX, probably two servers running.\n"));
 	exit(-1);
     }
-    rx_SetDestroyConnProc(tservice, (void (*)())h_FreeConnection);
     rx_SetMinProcs(tservice, 3);
     rx_SetMaxProcs(tservice, lwps);
     rx_SetCheckReach(tservice, 1);
